#include "stdafx.h"

//-- Global data 
const int MODE_VIEW = 0;
const int MODE_EDIT = 1;
const int MODE_NEW = 2;

char * g_szConfigTitle = "QUBIO Hardware Configuration";

const char * g_pchGainFormat="%5.1f";
const char * g_pchScaleFormat="%8.4f";

//-----------------------------------------------------------------------------
class CVwConfig : public CDialog { 
	public : 
		int m_iConfig;					// -1 = New 
		CConfiguration * m_pConfig;
		
		//-- Controls 
		CDlgEdit m_edtConfigName;
		CDlgCombo m_cmbDevice;
		
		CDlgCheck m_chkModeRSE;
		CDlgCheck m_chkModeNRSE;
		CDlgCheck m_chkModeDIFF;
		
		CDlgCheck m_achkAI[CONFIG_AIROWS];
		CDlgEdit m_aedtAIName[CONFIG_AIROWS];
		CDlgCombo m_acmbAIGain[CONFIG_AIROWS];
		CDlgEdit m_aedtAIUnits[CONFIG_AIROWS];
		CDlgEdit m_aedtAIScaling[CONFIG_AIROWS];

		CDlgCheck m_achkAO[CONFIG_AOROWS];
		CDlgEdit m_aedtAOName[CONFIG_AOROWS];
		CDlgEdit m_aedtAOUnits[CONFIG_AOROWS];
		CDlgEdit m_aedtAOScaling[CONFIG_AOROWS];
		
		CDlgEdit m_aedtDigoutNames[CONFIG_DOROWS];
		CDlgCombo m_selDOMode[CONFIG_DOROWS];
		CDlgCombo m_selDOModeAll;

		CDlgButton m_btnSave;
		CDlgButton m_btnCancel;
		CDlgButton m_btnDelete;
		CDlgButton m_btnCopy;
		
		//---- Constructor 
		CVwConfig( ) { 
			m_iConfig=-1;
			m_pConfig=NULL;
			m_iID = IDD_CONFIGVIEW;
			}

		//-- Initialize controls (Limit text entry length for edit boxes)
		void InitializeControls( ) { 
			int i;
			m_edtConfigName.SetHandle( GetDlgItem(m_hDlg, IDC_EDTNEWCONFIGNAME), "Unique name to identify the configuration." );
			m_edtConfigName.SetTextLimit( CONFIG_NAME_LENGTH );
			m_cmbDevice.SetHandle( GetDlgItem(m_hDlg, IDC_CMBDEVICE), "Select device name" );
			m_chkModeRSE.SetHandle( GetDlgItem(m_hDlg, IDC_SELMODE_RSE), "Referenced Single Ended Acquisition" );
			m_chkModeNRSE.SetHandle( GetDlgItem(m_hDlg, IDC_SELMODE_NRSE), "Non-referenced Single Ended Acquisition" );
			m_chkModeDIFF.SetHandle( GetDlgItem(m_hDlg, IDC_SELMODE_DIFF), "Differential Acquisition (ex. 8-0, 9-1, 10-2, 11-3, ...)" );

			//-- Analog acquisition
			for( i=0; i<CONFIG_AIROWS; ++i ) { 
				m_achkAI[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKAI00+i), "Toggle to include this acquisition channel");
				m_aedtAIName[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAINAME00+i), "Channel name for scope display");
				m_acmbAIGain[i].SetHandle( GetDlgItem(m_hDlg, IDC_CMBAIGAIN00+i), "Gain setting to multiply acquired signal prior to display");
				m_aedtAIUnits[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAIUNITS00+i), "Descriptive units for scope display");
				m_aedtAIScaling[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAISCALING00+i), "Conversion factor volts/unit.  Note:Gain affects analog signal; scaling affects digitized values.");

				m_aedtAIName[i].SetTextLimit( CONFIG_NAME_LENGTH );
				m_aedtAIUnits[i].SetTextLimit( CONFIG_NAME_LENGTH );
				m_aedtAIScaling[i].SetTextLimit( 16 );	
				}

			//-- analog generation
			for( i=0; i<CONFIG_AOROWS; ++i ) { 
				m_achkAO[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKAO00+i), "Toggle to include this generation channel");
				m_aedtAOName[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAONAME00+i), "Channel name for scope display");
				m_aedtAOUnits[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAOUNITS00+i), "Descriptive units for scope display");
				m_aedtAOScaling[i].SetHandle( GetDlgItem(m_hDlg, IDC_EDTAOSCALING00+i), "Conversion factor from protocol values to output volts");
				
				m_aedtAOName[i].SetTextLimit( CONFIG_NAME_LENGTH );
				m_aedtAOUnits[i].SetTextLimit( CONFIG_NAME_LENGTH );
				}

			//-- digital 
			for( i=0; i<CONFIG_DOROWS; ++i ) {
				m_aedtDigoutNames[i].SetHandle( GetDlgItem( m_hDlg, IDC_EDTDIGOUT1+i ), "Digital output name for data labels" );
				m_aedtDigoutNames[i].SetTextLimit( 8 );

				//m_achkAutoReset[i].SetHandle( GetDlgItem(m_hDlg, IDC_CHKAUTORESET1+i ), "Toggle whether digital signals pulse, then return to 0 or remain on(high) when clicked.");

				m_selDOMode[i].SetHandle( GetDlgItem(m_hDlg, IDC_DOMODE0+i ), "Select automatic shut off, manual shut off or protocol based on/off control");
				for( int iMode=0; iMode<5; ++iMode ) 
					m_selDOMode[i].AddString( g_apchDigMode[iMode] );
				}

			m_selDOModeAll.SetHandle( GetDlgItem(m_hDlg, IDC_CMBDOMODEALL ), "Select mode for digital lines");
			for( int iMode=0; iMode<5; ++iMode ) 
				m_selDOModeAll.AddString( g_apchDigMode[iMode] );
	
			//-- dialogue 
			m_btnSave.SetHandle( GetDlgItem(m_hDlg, IDC_BTNSAVE ), "Save changes to this configuration");
			m_btnCancel.SetHandle( GetDlgItem(m_hDlg, IDC_BTNCANCEL ), "Cancel changes to this configuration");
			m_btnDelete.SetHandle( GetDlgItem(m_hDlg, IDC_BTNDELCONFIG ), "Delete this configuration");
			m_btnCopy.SetHandle( GetDlgItem(m_hDlg, IDC_BTNCOPYCONFIG ), "Make a copy of this configuration");
			}

		//-----------------------------------------------------------------------------
		//-- Enable the controls for a single AI channel { name, gain , units, scaling }
		void EnableAIRow( int iRow ) {
			bool bEnable = (BST_CHECKED==IsDlgButtonChecked( m_hDlg, IDC_CHKAI00+iRow ));
			m_aedtAIName[iRow].SetEnabled( bEnable );
			m_acmbAIGain[iRow].SetEnabled( bEnable );
			m_aedtAIUnits[iRow].SetEnabled( bEnable );
			m_aedtAIScaling[iRow].SetEnabled( bEnable );
			}

		//-- Enable the controls for a single AO channel { name, units, scaling }
		void EnableAORow( int iRow ) {
			bool bEnable = (BST_CHECKED==IsDlgButtonChecked( m_hDlg, IDC_CHKAO00+iRow ));
			m_aedtAOName[iRow].SetEnabled( bEnable );
			m_aedtAOUnits[iRow].SetEnabled( bEnable );
			m_aedtAOScaling[iRow].SetEnabled( bEnable );
			}

		//-- Load device combo from NiCard() array 
		void FillDeviceList() { 
			m_cmbDevice.ClearContents();
			for( int i=0; i<NiCardCount(); ++i )
				m_cmbDevice.AddString( NiCard(i)->m_achName );
			}

		//-- look up selected card in list by name		
		int CurrentDevice( ) { 
			//-- get text of selected device name
			char achBufName[CNICard::NI_NAMESIZE+1];
			if( ! m_cmbDevice.GetText( achBufName, CNICard::NI_NAMESIZE ) ) 
				return -1;
			//-- Check if it is in list 
			return NiCardFind( achBufName );
			}

		//-----------------------------------------------------------------------------
		//-- Set contents of one Input channel row 
		bool SetupAIRow( int iRow, CChannelConfig * p ) {
			char buf[1024];
			m_achkAI[iRow].SetCheck( p->m_iActive==1 );					//-- Active
			m_aedtAIName[iRow].SetText( p->m_cChannelName.Buffer() );	//-- Name
			sprintf( buf, g_pchGainFormat, p->m_dGain );				//-- Gain
			m_acmbAIGain[iRow].SetCurSel( buf );
			m_aedtAIUnits[iRow].SetText( p->m_cUnits.Buffer() );		//-- Units
			m_aedtAIScaling[iRow].SetTextDouble( p->m_dScaling, g_pchScaleFormat );	//-- Scaling
			return true;
			}

		//-- Get contents of one Input channel row 
		bool TeardownAIRow( int iRow, CChannelConfig * p ) { 
			CString c;					//TODO - error checking on entries, messages to user
			p->m_iActive = m_achkAI[iRow].GetCheck();				//-- Active
			m_aedtAIName[iRow].GetText( &p->m_cChannelName );		//-- Name
			p->m_dGain = ( m_acmbAIGain[iRow].GetText( &c ) ? atof(c.Buffer()) : 1.0 );	//-- Gain
			m_aedtAIUnits[iRow].GetText( &p->m_cUnits );			//-- Units
			m_aedtAIScaling[iRow].GetText( &c );					//-- Scaling
			p->m_dScaling = atof( c.Buffer() );
			return true;
			}

		// Setup 1 control row { checkbox button, name, units, scaling }
		void SetupAORow( int iRow, CChannelConfig * p ) {
			m_achkAO[iRow].SetCheck( p->m_iActive==1 );					//Active
			m_aedtAOName[iRow].SetText( p->m_cChannelName.Buffer() );	//Name
			m_aedtAOUnits[iRow].SetText( p->m_cUnits.Buffer() );		//Units
			m_aedtAOScaling[iRow].SetTextDouble( p->m_dScaling, g_pchScaleFormat );	//-- Scaling
			}

		bool TeardownAORow( int iRow, CChannelConfig * p ) { 
			CString c;					//TODO - error checking on entries, messages to user
			p->m_iActive = m_achkAO[iRow].GetCheck();				//-- Active
			m_aedtAOName[iRow].GetText( &p->m_cChannelName );		//-- Name
			m_aedtAOUnits[iRow].GetText( &p->m_cUnits );			//-- Units
			m_aedtAOScaling[iRow].GetText( &c );					//-- Scaling
			p->m_dScaling = atof( c.Buffer() );
			return true;
			}

		void FillGainLists( CNICard * pCard ) { 
			int iCh, iGain;
			char achBuf[32];
			CString c;
			for( iCh=0; iCh<CONFIG_AIROWS; ++iCh ) {
				m_acmbAIGain[iCh].GetText( &c );
				m_acmbAIGain[iCh].ClearContents();
				if( pCard == NULL )
					m_acmbAIGain[iCh].AddString( "  1.0" );
				else
					for( iGain=0; iGain<pCard->m_iIGains; ++iGain ) {
						sprintf( achBuf, g_pchGainFormat, pCard->m_afIGain[iGain] );
						m_acmbAIGain[iCh].AddString( achBuf );
						}
				m_acmbAIGain[iCh].SetCurSel( c.Buffer() ); // attempt to restore previous setting
				}
			}
			
		bool SetupConfig( ) { 
			m_edtConfigName.SetText( m_pConfig->m_cConfigName.ConstBuffer() );
			if( CB_ERR==m_cmbDevice.SetCurSel( m_pConfig->m_cDeviceName.Buffer(), true) ) {
				}

			m_chkModeRSE.SetCheck( m_pConfig->m_iAcqMeasMode==0 );
			m_chkModeNRSE.SetCheck( m_pConfig->m_iAcqMeasMode==1 );
			m_chkModeDIFF.SetCheck( m_pConfig->m_iAcqMeasMode==2 );

			FillGainLists( NiCard(CurrentDevice()) );
			for( int iAI=0; iAI<CONFIG_AIROWS; ++iAI ) {
				SetupAIRow( iAI, &(m_pConfig->m_aoInChnCfg[iAI]) );
				EnableAIRow( iAI );
				}				
			for( int iAO=0; iAO<CONFIG_AOROWS; ++iAO ) {
				SetupAORow( iAO, &(m_pConfig->m_aoOutChnCfg[iAO]) );
				EnableAORow( iAO );
				}
			for( int i=0; i<CONFIG_DOROWS; ++i ) {
				m_aedtDigoutNames[i].SetText( m_pConfig->m_acDigName[i].Buffer() );
				m_selDOMode[i].SetCurSel( m_pConfig->m_aiDigMode[i] );
				}
			m_selDOModeAll.SetCurSel( -1 );
			return true;
			} 
			
		bool TeardownConfig( ) { 
			m_edtConfigName.GetText( &m_pConfig->m_cConfigName );
			if( ! m_cmbDevice.GetText( &m_pConfig->m_cDeviceName ) ) 
				return false;

			m_pConfig->m_iAcqMeasMode = m_chkModeDIFF.GetCheck() ? 2 : (m_chkModeNRSE.GetCheck() ? 1 : 0);
			 				
			for( int iAI=0; iAI<CONFIG_AIROWS; ++iAI ) 
				TeardownAIRow( iAI, &(m_pConfig->m_aoInChnCfg[iAI]) );
			for( int iAO=0; iAO<CONFIG_AOROWS; ++iAO ) 
				TeardownAORow( iAO, &(m_pConfig->m_aoOutChnCfg[iAO]) );
			for( int i=0; i<CONFIG_DOROWS; ++i ) {
				m_aedtDigoutNames[i].GetText( &(m_pConfig->m_acDigName[i]) );
				m_pConfig->m_aiDigMode[i] = m_selDOMode[i].GetCurSel( );
				}
			return true;
			} 

		//-- [SAVE] - 
		void OnBtnSave( ) { 
			//-- Validate name prior to saving
			CString cNewName;
			m_edtConfigName.GetText( &m_pConfig->m_cConfigName );
			if( m_iConfig==-1 ) {		
				// if new, check for existing of same name 
				if( g_oCfgs.FindConfig( cNewName.ConstBuffer() ) != -1 ) { 
					MessageBox( m_hDlg, "The configuration name is already in use.", g_szConfigTitle, MB_OK );
					return;
					}
				}
			else if( 0!=strcmp(cNewName.ConstBuffer(),m_pConfig->m_cConfigName.ConstBuffer())) { 
				//-- If edit and name changed, check for existing not at pos m_iConfig.
				if( g_oCfgs.FindConfig( cNewName.ConstBuffer() ) != -1 ) { 
					MessageBox( m_hDlg, "The configuration name is already in use.", g_szConfigTitle, MB_OK );
					return;
					}
				}

			//-- Copy values from controls to object				
			if( ! TeardownConfig( ) )
				return;
				
			//-- If new, append saved configuration to list
			if( m_iConfig==-1 )
				g_oCfgs.AddConfig( m_pConfig );
				
			//-- Save changes to disk and update dynamic menu.
			OnHardwareConfigUpdated( );
			
			DestroyWindow( GetParent( m_hDlg ) );
			}
			
		//-- [DELETE] - confirm, then remove current configuration
		void OnBtnDelete( ) { 
			if( MessageBox( m_hDlg, "Delete this configuration ?", g_szConfigTitle, MB_YESNO ) == IDYES ) {
				g_oCfgs.DeleteConfig( m_iConfig );
				OnHardwareConfigUpdated( );
				DestroyWindow( GetParent( m_hDlg ) );
				}
			
			}
				
		//-- [CANCEL]
		void OnBtnCancel( ) { 
			if( m_iConfig==-1 )		
				delete m_pConfig;
			DestroyWindow( GetParent( m_hDlg ) );
			}
			
		//-- [COPY]
		void OnBtnCopy() { 
			if( IDYES==MessageBox( GetParent(m_hDlg), "Create a copy of this configuration to modify", g_szConfigTitle, MB_YESNO ) ) {
				m_edtConfigName.SetText("");
				m_pConfig = new CConfiguration;
				m_iConfig = -1;
				TeardownConfig(  );		// unneeded - will eventually teardown if saved.
				}
			}
			
		//-- Create a new config	
		void OnNewConfig() {
			m_btnDelete.SetVisible( false );
			m_btnCopy.SetVisible( false );
			
			//-- create a new config 
			m_iConfig = -1;
			m_pConfig = new CConfiguration;

			//-- Default to first Device 
			if( m_cmbDevice.GetCount()>0 )  
				m_cmbDevice.SetCurSel(0);
				
			//-- Default to 2 active AI channels 
			m_pConfig->m_aoInChnCfg[0].m_iActive=1;
			m_pConfig->m_aoInChnCfg[1].m_iActive=1;

			//-- Display new config 		
			SetupConfig( ); 
			}

		//-- edit an existing config		
		void OnEditConfig( CConfiguration * pConfig , int iConfig ) { 
			m_iConfig=iConfig;
			m_pConfig=pConfig;
			SetupConfig( ); 
			}
			
		//-- select mode in extra digital combo - sets mode for all digitals
		void SetAllDOMode() { 
			int iSel = m_selDOModeAll.GetCurSel();
			if( iSel >= 0 ) 
				for( int i=0; i<CONFIG_DOROWS; ++i ) 
					m_selDOMode[i].SetCurSel( iSel );
			}
			
	};
	
//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK ConfigDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	CVwConfig * pVw;
	
	//-- create the CVwConfig to go with the dialog 
	if( iMsg == WM_INITDIALOG ) { 
		pVw = new CVwConfig;
		pVw->m_hDlg=hDlg;
		pVw->InitializeControls();
		pVw->FillDeviceList();
		SetWindowLong( GetParent(hDlg), 0, (long) pVw ) ;
		return TRUE;
		}
	
	//-- get the ViewClass, possibly NULL for initial messages 
	pVw = (CVwConfig *) GetWindowLong(  GetParent( hDlg ), 0 );
	
	//-- Message handlers 
	int wmId, wmEvent;
	switch( iMsg ) { 
		case WM_CONFIG_EDIT :
			pVw->OnEditConfig( (CConfiguration *)lParam, (int)wParam);
			break;
			
		case WM_CONFIG_NEW :
			pVw->OnNewConfig();
			break;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 

			//-- Toggle AI channel - enable or disable controls
			if( wmId >= IDC_CHKAI00 && wmId < IDC_CHKAI00+CONFIG_AIROWS )		 
				pVw->EnableAIRow( wmId - IDC_CHKAI00 );
				
			//-- Toggle AO channel - enable or disable controls
			if( wmId >= IDC_CHKAO00 && wmId < IDC_CHKAO00+CONFIG_AOROWS )	
				pVw->EnableAORow( wmId - IDC_CHKAO00 );

			switch( wmId ) { 
				case IDC_CMBDOMODEALL : 
					if( wmEvent==CBN_SELCHANGE ) 
						pVw->SetAllDOMode();
					break;

				case IDC_CMBDEVICE :			//-- {Device} Combo Box 
					if( wmEvent==CBN_SELCHANGE ) { 
						int iDev = pVw->CurrentDevice();
						if( iDev!=-1 )
							pVw->FillGainLists( NiCard(iDev) );
						}
					break;

				case IDC_BTNDELCONFIG :			//-- [DELETE]
					pVw->OnBtnDelete();
					break;

				case IDC_BTNSAVE :				//-- [SAVE]
					pVw->OnBtnSave( );
					break;

				case IDC_BTNCANCEL :			//-- [CANCEL]
					pVw->OnBtnCancel( );
					break;
					
				case IDC_BTNCOPYCONFIG : 
					pVw->OnBtnCopy();
					break;
				}
			return TRUE;

		case WM_PAINT : 
			/*{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hDlg, &ps);
			EndPaint(hDlg, &ps);
			}*/
			return false;

		case WM_DESTROY : 
			delete pVw;
			SetWindowLong( GetParent( hDlg ), 0, 0);
			WindowListRemove( hDlg );
			return TRUE;
			
		default : 
			break;
		}
		
	return FALSE;
	} 

